clearvars;close all;clc;
%Geometry 
geometry.L = 107.6; % beam length (m)
geometry.E = 2.1e11; % Young Modulus (Pa)
geometry.nu = 0.3; % Poisson ratio
geometry.rho = 8500; % density (kg/m^3)
geometry.t = 0.06 ; %thikness of the tube (m)

% case of a tube
D = 6; % beam diameter (m)
d = D - geometry.t ; % Inner diameter of the tube (m)
% beam is symmetrical around axes y and z
Iy = pi.*(D.^4 - d.^4)./64;
Ix = pi.*(D.^4 - d.^4)./64;
geometry.I = Iy; % affectation of quadratic moment

%Mass at the tip of the beam
geometry.Mtip = 400*10^(3) ; %(kg) 

ndisc = 100 ; % number of discretisation points for the beam
geometry.y = linspace(0,geometry.L ,ndisc);
V = ((pi.*(D/2).^2)-(pi.*(d/2).^2))*geometry.L;
M = V*geometry.rho ;
geometry.m = (geometry.rho.*V)./geometry.L;

% Number of modes
Nmodes =4; % number of mode wanted

[phi,wn] = eigenModes(geometry,Nmodes);

figure
ylabel('x [m]')
for ii=1:Nmodes
    subplot(1,Nmodes,ii)
    box on;grid on
    plot(phi(ii,:),geometry.y);
    xlabel(['\phi_',num2str(ii)])
    axis([-inf inf 0 107.6])
    if ii == 1
        ylabel('x [m]')
    end
    title(['w_',num2str(ii),' = ',num2str(wn(ii),3),' rad/s']);
end
set(gcf,'color','w')
    