function[x,f] = Joint_input_state_beam_func(mea,alt_d,alt_a)

t = linspace(0,100,125001);
dt = (t(end) - t(1))/(length(t)-1); %Time step%

%System parameters
%Geometry 
geometry.L = 107.6; % beam length (m)
geometry.E = 2.1e11; % Young Modulus (Pa)
geometry.nu = 0.3; % Poisson ratio
geometry.rho = 8500; % density (kg/m^3)
geometry.t = 0.06 ; %thikness of the tube (m)

% case of a tube
D = 6 ; % beam diameter (m)
d = D - geometry.t ; % Inner diameter of the tube (m)
% beam is symmetrical around axes y and z
Iy = pi.*(D.^4 - d.^4)./64;
Ix = pi.*(D.^4 - d.^4)./64;
geometry.I = Iy; % affectation of quadratic moment

%Mass at the tip of the beam
geometry.Mtip = 400*10^(3) ; %(kg) 

ndisc = 108 ; % number of discretisation points for the beam
geometry.y = linspace(0,geometry.L ,ndisc);
V = ((pi.*(D/2).^2)-(pi.*(d/2).^2))*geometry.L;
geometry.m = (geometry.rho.*V)./geometry.L;


% Number of modes
Nmodes =3; % number of mode wanted
[phi,wn] = eigenModes(geometry,Nmodes);
%Eigen frequencies
Omega = diag(wn);
%Damping ratio
Gamma = 2/100*diag([1.89 1.38 0.89]).*Omega;

%Excitation force location
np = 1; %Number of forces applying on the structure
Sp = zeros(ndisc, np);
Sp(ndisc,1) = 1; %1 force located on the tip of the beam
%Measurement transmission matrices
nd = 2; %number of measurement (disp, vel and acc)
Sdd = zeros(nd,ndisc);
Sdd(1,alt_d) = 1 ; %Displacement measurement taken at 20 m from the bottom
Sdv = zeros(nd,ndisc); %No velocity measurement
Sda = zeros(nd,ndisc);
Sda(2,alt_a) = 1 ; %Acceleration measurement taken at 20 m from the tip

%State space model
ns = 2*Nmodes;
A = exp([zeros(Nmodes) eye(Nmodes); -Omega.^2 -Gamma]*dt);
B = (A - eye(ns))/[zeros(Nmodes) eye(Nmodes); -Omega.^2 -Gamma]*[zeros(Nmodes,np); phi*Sp];
G = [Sdd*phi' - Sda*phi'*Omega^2 , Sdv*phi' - Sda*phi'*Gamma];
J = (Sda*(phi'*phi)*Sp); 

%Stochastic forces
nps = ndisc; %Number of stochastic forces
ps = ones(nps,1); %Stochastic forces distributed all along the beam
ps(1:19) = 1000*ones(19,1);
ps(20) = 5000;
ps(21:nps) = 100*ones(nps-20,1);
Cps = diag(ps).^2; %Stochastic forces of 10 N standart deviation
Bs = zeros(ns,nps);
Js = zeros(nd,nps);
for i=1:nps
    Sps = zeros(nps,1);
    Sps(i) = ps(i);
    Bs(:,i)=(A - eye(ns))/[zeros(Nmodes) eye(Nmodes); -Omega.^2 -Gamma]*[zeros(Nmodes,np); phi*Sps];
    Js(:,i)=(Sda*(phi'*phi)*Sps);
end

%error covariance matrices
Rm = [10^(-1) 0; 0 10^(-1)]; %measurement error on displacement and acceleration 
Q = Bs*Cps*Bs';
S = Bs*Cps*Js' ;
R = Js*Cps*Js' + Rm ;

%Output vector
nde = 1; %Number of output to be estimated
Sdde = zeros(nde,ndisc);
Sdde(1,100) = 1 ; %Displacement measurement taken at the tip of the beam
Sdve = zeros(nde,ndisc); %No velocity output
Sdae = zeros(nde,ndisc); %No acceleration output
Ge = [Sdde*phi' - Sdae*phi'*Omega^2 , Sdve*phi' - Sdae*phi'*Gamma];
Je = (Sdae*(phi'*phi)*Sp);
Jes = zeros(nde,nps); %Relating output vector to stochastic forces

%output covariance matrices
Re = Jes*Cps*Jes';
Rc = Jes*Cps*Js';

%Pre-definition of variables
x = zeros(ns,length(mea));
x_e = zeros(ns,length(mea));
Px = zeros(ns,ns);
Pp = zeros(np,np); %size of J matrix
f = zeros(np,length(mea));
de = zeros(nde,length(mea));

%Initialization
y0 = phi*mea(1,1)*ones(ndisc,1); %Initial position (modal coordinates) for each element 
v0 = phi*mea(2,1)*ones(ndisc,1); %Initial velocity (modal coordinates) for each element
Px0 = zeros(size(Px));
x(:,1) = [y0; v0]; 
Px = Px0;
Px_e = Px;
Rk_test = zeros(2,2,length(mea));
Px_test = zeros(ns,ns,length(mea));
Pp_test = zeros(np,np,length(mea));
Pxp_test = zeros(ns,np,length(mea));

%Looping process 
for i=2:length(mea)
    %Input estimation 
    R_k = G*Px_e*G'+R;
    Rk_test(:,:,i) = R_k;
    M_k = (J'/R_k*J)\(J'/R_k);
    f(i) = M_k*(mea(:,i)-G*x(:,i-1));
    Pp = eye(size(Pp))/(J'/R_k*J);
    Pp_test(:,:,i) = Pp;
    %Measurement update
    K = Px_e*G'/R_k;
    x_e(:,i) = x(:,i-1)+K*(mea(:,i)-G*x(:,i-1)-J*f(i)); 
    Px = Px_e-K*(R_k-J*Pp*J')*K';
    Px_test(:,:,i) = Px;
    Pxp = -K*J*Pp;
    Pxp_test(:,:,i) = Pxp;
    %Time update 
    x(:,i) = A*x_e(:,i)+B*f(i);
    I=eye(size(J*M_k));
    N_k = A*K*(I-J*M_k)+B*M_k;
    Px_e = [A B]*[Px Pxp;Pxp' Pp]*[A';B']+Q+N_k*S'+S*N_k';
   
    %Output vector
    de(i) = Ge*x_e(:,i)+Je*f(i);
    Z = Ge*K*(I-J*M_k)+Je*M_k;
    Pde = Ge*Px*Ge'+Je*Pp*Je'+Ge*Pxp*Je'+Je*Pxp'*Ge'+Re-Z*Rc'-Rc*Z';
end
