%Kalman filter algorithm tracking a single dof system and estimating
mea = [];
%Measurements%
measures_dis_vel = xlsread('measures.xlsx')';
mea(1,:) = measures_dis_vel(1,:);
mea(2,:) = xlsread('mea_accel.xlsx');
%Time%
t = linspace(0,10,length(mea));
dt = (t(end) - t(1))/(length(t)-1); %Time step%
% Noise in the measurements
sigma_dis = 10^(-3);
av_dis = 0;
sigma_acc = 10^(-2);
av_acc = 0;
Vm(1,:) = sigma_dis*randn(1,length(mea));
Vm(2,:) = sigma_acc*randn(1,length(mea));
mea = mea + Vm;

% System parameters %
m = 10; %[kg]%
k = 1000; %[N/m]%
c = 2.0; %[kg/s]%

%Define matrices
Rm = [(sigma_dis)^2 0; 0 (sigma_acc)^2]; % [(m/s^2)^2] measure uncertainty
Cp = (2)^2; %Covariance matrix (stochastic force with standart deviation of 2 N)
%State matrices
A = [1 dt; -dt*k/m 1-dt*c/m];
B = [0; dt/m];
J = [0; 1/m];
G = [1 0 ;-k/m -c/m];
size_B = size(B);
size_A = size(A);
np = size_B(2);
ns = size_A(2);
%Stochastic excitation matrix
Bs = [0; dt/m];
Js = [0; 1/m];
%Output quantity matrices
Ge = [1 0]; 
Je = 0;
Jes = 0;%Matrix relating extrapolated output vector to vector of sthocastique forces
% Noise processes covariance matrices
n = length(B);
l = length(J);
COV = [Bs;Js]*Cp*[Bs' Js'];
Q = COV(1:n,1:n);
S = COV(1:n,n+1:length(COV));
R = COV(n+1:length(COV),n+1:length(COV)) + Rm;
Re = Jes*Cp*Jes';
Rc = Jes*Cp*Js';

%Pre-definition of variables
x = zeros(n,length(mea));
x_e = zeros(n,length(mea));
Px = zeros(n); %size of B matrix 
Pp = zeros(length(J(1,:))); %size of J matrix
f = zeros(1,length(mea));
de = zeros(1,length(mea));

%Initialization
y0 = 0; %Initial position
v0 = 0; %Initial velocity
Px0 = [0 0;0 0];
x(:,1) = [y0; v0]; 
Px = Px0;
Px_e = Px;

%Looping process 
for i=2:length(mea)
    %Input estimation 
    R_k = G*Px_e*G'+R;
    M_k = (J'/R_k*J)\(J'/R_k);
    f(i) = M_k*(mea(:,i)-G*x(:,i-1));
    Pp = eye(size(Pp))/(J'/R_k*J);    
    %Measurement update
    K = Px_e*G'/R_k;
    x_e(:,i) = x(:,i-1)+K*(mea(:,i)-G*x(:,i-1)-J*f(i)); 
    Px = Px_e-K*(R_k-J*Pp*J')*K';
    Pxp = -K*J*Pp;
    %Time update 
    x(:,i) = A*x_e(:,i)+B*f(i);
    I=eye(size(J*M_k));
    N_k = A*K*(I-J*M_k)+B*M_k;
    Px_e = [A B]*[Px Pxp;Pxp' Pp]*[A';B']+Q+N_k*S'+S*N_k';
    %Output vector
    de(i) = Ge*x_e(:,i)+Je*f(i);
    Z = Ge*K*(I-J*M_k)+Je*M_k;
    Pde = Ge*Px*Ge'+Je*Pp*Je'+Ge*Pxp*Je'+Je*Pxp'*Ge'+Re-Z*Rc'-Rc*Z';
end

