t = linspace(0,10,10001);
dt = median(diff(t));

% System parameters %
m = 10; %[kg]%
k = 1000; %[N/m]%
c = 2; %[kg/s]%
w0 = 7; %[rad/s]%

% Applied force %
F0 = 20 ; %[N]%
F = F0*cos(w0*t) + F0/2*cos(12*t) + F0/10*cos(10*t) +  F0/3*cos(20*t) + 1*randn(1,length(t));

% Initial conditions %
y0 = 0;
v0 = 0;

% State of system %
[y] = CentDiff(F,m,k,c,dt,y0,v0); %displacement
v = zeros(1,length(y));
for i=2:length(y)-1
    v(i)=(y(i+1)-y(i-1))/(2*dt); %Velocity
end
v(length(y))= (y(length(y))-y(length(y)-1))/dt;
acc = (1/m)*(F - c*v - k*y);

xlswrite('measures.xlsx',[y', v']);
xlswrite('mea_accel.xlsx',acc);

figure(1)
plot(t,y,'r')
%hold on
%plot(t,v,'b')
%plot(t,acc,'g--')
%hold off

%Frequency analysis
L = length(t); %Length of the signal
Fs = L/t(end); %Sampling frequency
t2 = (0:(L-1))/Fs;
Y = fft(y);
P2 = abs(Y/L);
P1 = P2(1:(L/2)+1);
P1(2:end-1) = 2*P1(2:end-1);
f = Fs*(0:(L/2))/(L);
omega = 2*pi*f;
figure(2)
plot(omega,P1)
title('spectrum of displacement signal')
xlabel('frequency [Hz]')
ylabel('amplitude [m]')


