function [phi,wn,M] = eigenModes(geometry,Nmodes)

y= geometry.y;
Nz = numel(y); % number of discrete elements

% volume V and mass m of the beam
m = geometry.m;
L = geometry.L;

D = 6 ; % beam diameter (m)
d = D - geometry.t ; % Inner diameter of the tube (m)
Mtip = 700000 ; 
%Mass matrix for each element
m_el = (1/2)*geometry.rho*((pi.*(D/2).^2)-(pi.*(d/2).^2))*geometry.L/Nz*[1 0;0 1];
%Mass matrix
M = zeros(Nz);
for i = 1:Nz-1
    M(i:i+1,i:i+1) = M(i:i+1,i:i+1) + m_el ;
end
M(Nz,Nz)=M(Nz,Nz)+Mtip;

% get the non trivial solution of f

if Nmodes<10,
    Ndummy=1:Nmodes^2; % the number is arbitrary fixed as the square of the number of Nmodes.
else
    Ndummy=1:100; % the number is arbitrary fixed as the square of the number of Nmodes.
end
tolX = 1e-8;
tolFun = 1e-8;
options=optimset('TolX',tolX,'TolFun',tolFun,'Display','off');
h =fsolve(@modeShape,Ndummy,options);
% small values of h come from numerical errors
h(h<0.4)=[];
% Analytically, many solutions are identical to each other, but numerically, it is not the
% case. Therefore I need to limit the prceision of the solutions to 1e-4.
h = round(h*1e4).*1e-4;
% the uniques solution are called beta:
beta = unique(h);
beta = beta(1:min(numel(beta),Nmodes));

%modes shapes calculations
wn = beta.^2.*sqrt(geometry.E*geometry.I./(m.*geometry.L^4)); % in rad
phi = zeros(Nmodes,Nz);

    for ii=1:Nmodes
        phi(ii,:)=(cosh(beta(ii)*y./L)-cos(beta(ii)*y./L))+...
                ((cosh(beta(ii))+cos(beta(ii))).*(sin(beta(ii)*y./L)-sinh(beta(ii)*y./L)))./...
                (sin(beta(ii))+sinh(beta(ii)));
            
        phi(ii,:)=phi(ii,:)/sqrt((phi(ii,:)*M*phi(ii,:)')); %Mass normalization
    end
    
end


